@@ -2,6 +2,8 @@ import os
 import logging
 import json
 
+from operator import itemgetter
+
 import dtoolcore
 import dtoolcore.utils
 
@@ -9,6 +11,7 @@ from ruamel.yaml import YAML
 
 # This is a hack.
 from dtool_info.inventory import _dataset_info
+from dtool_info.utils import sizeof_fmt
 
 from dtool_gui_tk.metadata import MetadataSchemaItem
 
@@ -23,8 +26,8 @@ def get_json_schema_type(obj):
 
     :param obj: object to return the JSON schema type for
     :returns: JSON schema type as a string
-    :raises: :class:`dtool_gui_tk.models.UnsupportedTypeError` if the value is a
-             complex data structure. Currently supported data types are
+    :raises: :class:`dtool_gui_tk.models.UnsupportedTypeError` if the value is
+             a complex data structure. Currently supported data types are
              ``str``, ``int``, ``float``, and ``bool``.
     """
     if isinstance(obj, str):
@@ -573,6 +576,19 @@ class DataSetModel(object):
         self._dataset = dtoolcore.DataSet.from_uri(uri)
         self._metadata_model = metadata_model_from_dataset(self._dataset)
 
+    def get_item_props_list(self):
+        """Return list of dict of properties for each item in the dataset."""
+        item_props_list = []
+        for identifier in self._dataset.identifiers:
+            props = self._dataset.item_properties(identifier)
+            item_props_list.append({
+                "identifier": identifier,
+                "relpath": props["relpath"],
+                "size_int": props["size_in_bytes"],
+                "size_str": sizeof_fmt(props["size_in_bytes"])
+            })
+        return sorted(item_props_list, key=itemgetter("relpath"))
+
     def update_name(self, name):
         """Update the name of the dataset.
 
@@ -588,8 +604,8 @@ class DataSetModel(object):
 
         Both the dataset readme and annotations are updated.
 
-        :raises dtool_gui_tk.models.MetadataValidationError: if the metadata value
-            is not valid according to its schema
+        :raises dtool_gui_tk.models.MetadataValidationError: if the metadata
+            value is not valid according to its schema
         :raises dtool_gui_tk.models.MissingRequiredMetadataError: if a required
             metadata value has not been set
         """
@@ -665,8 +681,8 @@ class ProtoDataSetModel(object):
     def metadata_model(self):
         """Return the metadata model.
 
-        :returns: :class:`dtool_gui_tk.models.MetadataModel` instance or None if
-                  it has not been set
+        :returns: :class:`dtool_gui_tk.models.MetadataModel` instance or None
+                  if it has not been set
         """
         return self._metadata_model
 
@@ -728,8 +744,8 @@ class ProtoDataSetModel(object):
 
         :raises dtool_gui_tk.models.MissingInputDirectoryError: if the input
             directory has not been set
-        :raises dtool_gui_tk.models.MissingDataSetNameError: if the dataset name
-            has not been set.
+        :raises dtool_gui_tk.models.MissingDataSetNameError: if the dataset
+            name has not been set.
         :raises dtool_gui_tk.models.MissingBaseURIModelError: if the base URI
             model has not been set.
         :raises dtool_gui_tk.models.MissingMetadataModelError: if the metadata
@@ -794,6 +810,8 @@ class DataSetListModel(object):
     def __init__(self):
         self._base_uri_model = None
         self._datasets = []
+        self._datasets_info = []
+        self._active_index = None
 
     @property
     def base_uri(self):
@@ -805,6 +823,10 @@ class DataSetListModel(object):
             return None
         return self._base_uri_model.get_base_uri()
 
+    @property
+    def active_index(self):
+        return self._active_index
+
     @property
     def names(self):
         """Return list of dataset names.
@@ -822,24 +844,62 @@ class DataSetListModel(object):
         if self._base_uri_model.get_base_uri() is not None:
             self.reindex()
 
-    def get_uri(self, index):
-        """Return the URI of the dataset at a specific index in the list.
-
-        :param index: position of dataset in list
+    def get_active_uri(self):
+        """Return the URI of the dataset at the active index.
         """
-        return self._datasets[index].uri
+        if self.active_index is None:
+            return None
+        return self._datasets[self.active_index].uri
+
+    def get_active_name(self):
+        """Return the name of the dataset at the active index.
+        """
+        if self.active_index is None:
+            return None
+        return self._datasets[self.active_index].name
+
+    def set_active_index(self, index):
+        """Set the active_index.
+
+        :raises: IndexError if the index is invalid
+        """
+        if len(self._datasets) == 0:
+            # No datasets in the model.
+            raise(IndexError())
+        if index < 0:
+            # Can't have a negative index.
+            raise(IndexError())
+        if index >= len(self._datasets):
+            raise(IndexError())
+        self._active_index = index
 
     def reindex(self):
         """Index the base URI."""
         self._datasets = []
+        self._datasets_info = []
+        self._active_index = None
         base_uri = self._base_uri_model.get_base_uri()
         if base_uri is None:
             return
         for ds in dtoolcore.iter_datasets_in_base_uri(base_uri):
             self._datasets.append(ds)
+            self._datasets_info.append(_dataset_info(ds))
+
+        # The initial active index is 0 if there are datasets in the model.
+        if len(self._datasets) > 0:
+            self._active_index = 0
+
+    def sort(self, key="name", reverse=False):
+        """Sort the datasets by items properties."""
+        logger.info("Sorting using key={}, reverse={}".format(key, reverse))
+        assert key in ("name", "size_int", "num_items", "creator", "date")
+        sort_values = [p[key] for p in self._datasets_info]
+        zipped_lists = zip(sort_values, self._datasets, self._datasets_info)
+        sorted_pairs = sorted(zipped_lists, key=itemgetter(0), reverse=reverse)
+        tuples = zip(*sorted_pairs)
+        _, self._datasets, self._datasets_info = [list(t) for t in tuples]
 
     def yield_properties(self):
         """Return iterable that yields dictionaries with dataset properties."""
-        for ds in self._datasets:
-            # This is a hack.
-            yield _dataset_info(ds)
+        for info in self._datasets_info:
+            yield info
