@@ -14,6 +14,8 @@ import tkinter.ttk as ttk
 import tkinter.filedialog as fd
 import tkinter.messagebox as mb
 
+from idlelib.tooltip import Hovertip
+
 from dtool_gui_tk.models import (
     LocalBaseURIModel,
     DataSetListModel,
@@ -65,6 +67,9 @@ class DataSetListFrame(ttk.Frame):
         super().__init__(master)
         logger.info("Initialising {}".format(self))
 
+        self._sort_key = None
+        self._reverse_sort_order = False
+
         # Make sure that the GUI expands/shrinks when the window is resized.
         self.columnconfigure(0, weight=1)
         self.rowconfigure(0, weight=1)
@@ -78,11 +83,11 @@ class DataSetListFrame(ttk.Frame):
             selectmode="browse",
             columns=self.columns
         )
-        self.dataset_list.heading("name", text="Dataset name")
-        self.dataset_list.heading("size_str", text="Size")
-        self.dataset_list.heading("num_items", text="Num items")
-        self.dataset_list.heading("creator", text="Creator")
-        self.dataset_list.heading("date", text="Date")
+        self.dataset_list.heading("name", text="Dataset name", command=self.sort_by_name)  # NOQA
+        self.dataset_list.heading("size_str", text="Size", command=self.sort_by_size)  # NOQA
+        self.dataset_list.heading("num_items", text="Num items", command=self.sort_by_num_items)  # NOQA
+        self.dataset_list.heading("creator", text="Creator", command=self.sort_by_creator)  # NOQA
+        self.dataset_list.heading("date", text="Date", command=self.sort_by_date)  # NOQA
         self.dataset_list.column("size_str", width=80, anchor="e")
         self.dataset_list.column("num_items", width=80, anchor="e")
         self.dataset_list.column("creator", width=80, anchor="w")
@@ -99,33 +104,63 @@ class DataSetListFrame(ttk.Frame):
         # Bind event when row is selected.
         self.dataset_list.bind(
             "<<TreeviewSelect>>",
-            self.update_selected_dataset
+            self.update_selected_dataset_event
         )
 
-        # Layout the fame.
+        # Layout the frame.
         self.dataset_list.grid(row=0, column=0, sticky="nswe")
         yscrollbar.grid(row=0, column=1, sticky="ns")
 
         self.refresh()
 
-    def update_selected_dataset(self, event):
+    def _sort(self, sort_key="name"):
+        if self._sort_key == sort_key:
+            self._reverse_sort_order = not self._reverse_sort_order
+        else:
+            self._sort_key = sort_key
+            self._reverse_sort_order = False
+        self.root.dataset_list_model.sort(self._sort_key, self._reverse_sort_order)  # NOQA
+        self.refresh(reindex=False)
+
+    def sort_by_name(self):
+        self._sort("name")
+
+    def sort_by_size(self):
+        self._sort("size_int")
+
+    def sort_by_num_items(self):
+        self._sort("num_items")
+
+    def sort_by_creator(self):
+        self._sort("creator")
+
+    def sort_by_date(self):
+        self._sort("date")
+
+    def update_selected_dataset_event(self, event):
         selected = self.dataset_list.selection()[0]
         index = self.dataset_list.index(selected)
-        dataset_uri = self.root.dataset_list_model.get_uri(index)
+        self.update_selected_dataset(index)
+
+    def update_selected_dataset(self, index):
+        self.root.dataset_list_model.set_active_index(index)
+        dataset_uri = self.root.dataset_list_model.get_active_uri()
         self.root.load_dataset(dataset_uri)
         self.root.dataset_frame.refresh()
 
-    def refresh(self):
+    def refresh(self, reindex=True):
         """Refresh list dataset frame."""
         logger.info("Refreshing {}".format(self))
         self.dataset_list.delete(*self.dataset_list.get_children())
-        self.root.dataset_list_model.reindex()
+        if reindex:
+            self.root.dataset_list_model.reindex()
         for props in self.root.dataset_list_model.yield_properties():
             values = [props[c] for c in self.columns]
             self.dataset_list.insert("", "end", values=values)
-            logger.info("Loaded dataset: {}".format(props["name"]))
-        if len(self.root.dataset_list_model.names) > 0:
-            dataset_uri = self.root.dataset_list_model.get_uri(0)
+            logger.info("Loaded dataset: {}".format(props))
+
+        dataset_uri = self.root.dataset_list_model.get_active_uri()
+        if dataset_uri is not None:
             self.root.load_dataset(dataset_uri)
         else:
             self.root.dataset_model.clear()
@@ -140,14 +175,124 @@ class DataSetFrame(ttk.Frame):
 
         # Make sure that the GUI expands/shrinks when the window is resized.
         self.columnconfigure(0, weight=1)
-        self.rowconfigure(0, weight=1)
+        self.rowconfigure(3, weight=1)
+
+        self.dataset_title_frame = DataSetTitleFrame(self, root)
+        self.dataset_title_frame.grid(row=0, column=0, sticky="ew")
+
+        self.dataset_metadata_frame = DataSetMetadataFrame(self, root)
+        self.dataset_metadata_frame.grid(row=2, column=0, sticky="news")
+
+        self.dataset_item_frame = DataSetItemsFrame(self, root)
+        self.dataset_item_frame.grid(
+            row=3, column=0, sticky="news", pady=(4, 0)
+        )
 
         self.root = root
         if self.root.base_uri_model.get_base_uri() is not None:
             self.refresh()
 
     def refresh(self):
-        """Refreshing dataset frame."""
+
+        self.dataset_title_frame.refresh()
+        self.dataset_metadata_frame.refresh()
+        self.dataset_item_frame.refresh()
+
+
+class DataSetTitleFrame(ttk.Frame):
+    """View dataset title."""
+
+    def __init__(self, master, root):
+        super().__init__(master)
+        logger.info("Initialising {}".format(self))
+
+        self.root = root
+
+        # Make sure that the GUI expands/shrinks when the window is resized.
+        self.columnconfigure(0, weight=1)
+        self.rowconfigure(0, weight=1)
+
+        self.name = ttk.Label(self)  # NOQA
+        self.name.grid(row=0, column=0, sticky="ew")
+
+        ttk.Separator(self, orient=tk.HORIZONTAL).grid(
+            row=1, column=0, sticky="ew", pady=(0, 4)
+        )
+
+    def refresh(self):
+        self.name.config(text=self.root.dataset_model.name)
+
+
+class DataSetItemsFrame(ttk.Frame):
+    """View dataset items."""
+
+    def __init__(self, master, root):
+        super().__init__(master)
+        logger.info("Initialising {}".format(self))
+
+        # Make sure that the GUI expands/shrinks when the window is resized.
+        self.columnconfigure(0, weight=1)
+        self.rowconfigure(0, weight=1)
+
+        self.columns = ("relpath", "size_str")
+        self.item_list = ttk.Treeview(
+            self,
+            show="headings",
+            height=5,
+            columns=self.columns
+        )
+        self.item_list.heading("relpath", text="Relpath")
+        self.item_list.heading("size_str", text="Size")
+        self.item_list.column("relpath", width=300, anchor="w")
+        self.item_list.column("size_str", width=60, anchor="e")
+
+        # Add a scrollbar.
+        yscrollbar = ttk.Scrollbar(
+            self,
+            orient=tk.VERTICAL,
+            command=self.item_list.yview
+        )
+        self.item_list.configure(yscroll=yscrollbar.set)
+
+        # Layout the frame.
+        self.item_list.grid(row=0, column=0, sticky="nswe")
+        yscrollbar.grid(row=0, column=1, sticky="ns")
+
+        self.root = root
+        if self.root.base_uri_model.get_base_uri() is not None:
+            self.refresh()
+
+    def refresh(self):
+        """Refreshing dataset metadata frame."""
+        logger.info("Refreshing {}".format(self))
+        self.item_list.delete(*self.item_list.get_children())
+
+        for props in self.root.dataset_model.get_item_props_list():
+            values = [props["relpath"], props["size_str"]]
+            self.item_list.insert("", "end", values=values)
+
+        # Skip if a dataset is not loaded.
+        if self.root.dataset_model.name is None:
+            return
+
+
+class DataSetMetadataFrame(ttk.Frame):
+    """View dataset metadata."""
+
+    def __init__(self, master, root):
+        super().__init__(master)
+        logger.info("Initialising {}".format(self))
+
+        # Make sure that the GUI expands/shrinks when the window is resized.
+        self.columnconfigure((0, 1), weight=1)
+#       self.rowconfigure(0, weight=1)
+
+        self.root = root
+        if self.root.base_uri_model.get_base_uri() is not None:
+            self.refresh()
+
+    def refresh(self):
+        """Refreshing dataset metadata frame."""
         logger.info("Refreshing {}".format(self))
         for widget in self.winfo_children():
             logger.info("Destroying widget: {}".format(widget))
@@ -157,25 +302,28 @@ class DataSetFrame(ttk.Frame):
         if self.root.dataset_model.name is None:
             return
 
-        # Display the name.
-        name = ttk.Label(self, text=self.root.dataset_model.name)  # NOQA
-        name.grid(row=0, column=0, columnspan=2, sticky="nw")
-        separator = ttk.Separator(self, orient=tk.HORIZONTAL)
-        separator.grid(row=1, column=0, columnspan=2, sticky="ew")
-
         if self.root.dataset_model.metadata_model is None:
             ttk.Label(self, text="Dataset contains unsupported metadata types").grid(row=2, column=0, columnspan=2, sticky="ew")  # NOQA
 
         else:
-            for i, name in enumerate(self.root.dataset_model.metadata_model.in_scope_item_names):  # NOQA
-                value = self.root.dataset_model.metadata_model.get_value(name)
+            for i, name in enumerate(
+                self.root.dataset_model.metadata_model.required_item_names
+                + self.root.dataset_model.metadata_model.optional_item_names
+            ):
+
+                # Get the value from the dataset if it has been set.
+                try:
+                    value = self.root.dataset_model._dataset.get_annotation(name)  # NOQA
+                except dtoolcore.DtoolCoreKeyError:
+                    # A metadata item has not been set.
+                    continue
                 value_as_str = str(value)
                 row = i + 2
-                ttk.Label(self, text=name).grid(row=row, column=0, sticky="e")  # NOQA
+                ttk.Label(self, text=name).grid(row=row, column=0, sticky="ew")  # NOQA
                 entry = ttk.Entry(self, width=50)
                 entry.insert(0, value_as_str)
                 entry.configure(state="readonly")
-                entry.grid(row=row, column=1, sticky="w")  # NOQA
+                entry.grid(row=row, column=1, sticky="ew")  # NOQA
 
 
 class NewDataSetConfigFrame(ttk.Frame):
@@ -186,8 +334,13 @@ class NewDataSetConfigFrame(ttk.Frame):
         logger.info("Initialising {}".format(self))
         self.master = master
         self.root = root
+
+        # Make sure that the GUI expands/shrinks when the window is resized.
+        self.columnconfigure(0, weight=1)
+
         self.label_frame = ttk.LabelFrame(self, text="New dataset configuration")  # NOQA
-        self.label_frame.grid(row=0, column=0, sticky="nsew")
+        self.label_frame.grid(row=0, column=0, sticky="ew")
+        self.label_frame.columnconfigure(1, weight=1)
         self.refresh()
 
     def _validate_name_callback(self, name):
@@ -208,6 +361,7 @@ class NewDataSetConfigFrame(ttk.Frame):
 
         vcmd = (self.master.register(self._validate_name_callback), "%P")
         lbl = ttk.Label(self.label_frame, text="Dataset Name")
+        Hovertip(lbl, "Maximum 80 characters. Allowed characters: 0-9 a-z A-Z - _ .")  # NOQA
         entry = ttk.Entry(
             self.label_frame,
             validate="key",
@@ -240,6 +394,7 @@ class NewDataSetConfigFrame(ttk.Frame):
 
     def _setup_input_directory_field(self, row):
         lbl = ttk.Label(self.label_frame, text="Input data directory")
+        Hovertip(lbl, "Directory with data that should be included in the dataset.")  # NOQA
         logger.info("Current input directory: {}".format(
             self.master.proto_dataset_model.input_directory
         ))
@@ -280,8 +435,8 @@ class NewDataSetConfigFrame(ttk.Frame):
         index = schema_selection.index("basic")
         self.metadata_schema_combobox.current(index)
 
-        lbl.grid(row=row, column=0)
-        self.metadata_schema_combobox.grid(row=row, column=1)
+        lbl.grid(row=row, column=0, sticky="e")
+        self.metadata_schema_combobox.grid(row=row, column=1, sticky="ew")
 
     def _select_metadata_schema(self, event):
         widget = event.widget
@@ -311,10 +466,17 @@ class OptionalMetadataFrame(ttk.Frame):
     def __init__(self, master, root):
         super().__init__(master)
         logger.info("Initialising {}".format(self))
+
+        # Make sure that the GUI expands/shrinks when the window is resized.
+        self.columnconfigure(0, weight=1)
+        self.rowconfigure(0, weight=1)
+
         self.master = master
         self.root = root
         self.label_frame = ttk.LabelFrame(self, text="Optional metadata")  # NOQA
-        self.label_frame.grid(row=0, column=0)
+        self.label_frame.grid(row=0, column=0, sticky="nwse")
+        self.label_frame.columnconfigure(0, weight=1)
+        self.label_frame.rowconfigure(0, weight=1)
 
         self.optional_metadata_listbox = tk.Listbox(self.label_frame)
         self.optional_metadata_listbox.bind(
@@ -347,11 +509,19 @@ class MetadataFormFrame(ttk.Frame):
     def __init__(self, master, root):
         super().__init__(master)
         logger.info("Initialising {}".format(self))
+
+        # Make sure that the GUI expands/shrinks when the window is resized.
+        self.columnconfigure(0, weight=1)
+        self.rowconfigure(0, weight=1)
+
         self.master = master
         self.root = root
         self.entries = {}
+
         self.label_frame = ttk.LabelFrame(self, text="Metadata form")  # NOQA
         self.label_frame.grid(row=0, column=0, sticky="nsew")
+        self.label_frame.columnconfigure(1, weight=1)
+
         self.refresh()
 
     @property
@@ -388,11 +558,11 @@ class MetadataFormFrame(ttk.Frame):
         e.name = name
         e.bind("<<ComboboxSelected>>", self.value_update_event_focus_next)
         e.bind("<Return>", self.value_update_event_focus_next)
-        e.grid(row=row, column=1, sticky="ew")
+        e.grid(row=row, column=1, sticky="e")
         self.entries[name] = e
 
     def setup_entry_input_field(self, row, name, value):
-        e = tk.Entry(self.label_frame)
+        e = ttk.Entry(self.label_frame)
         if value is not None:
             e.insert(0, str(value))
         e.name = name
@@ -409,7 +579,7 @@ class MetadataFormFrame(ttk.Frame):
         _set_combobox_default_selection(e, values, value)
         e.name = name
         e.bind("<<ComboboxSelected>>", self.value_update_event_focus_next)
-        e.grid(row=row, column=1, sticky="ew")
+        e.grid(row=row, column=1, sticky="new")
         self.entries[name] = e
 
     def setup_input_field(self, row, name):
@@ -421,7 +591,7 @@ class MetadataFormFrame(ttk.Frame):
         if name in self.metadata_model.required_item_names:
             display_name = name + "*"
 
-        lbl = tk.Label(self.label_frame, text=display_name)
+        lbl = ttk.Label(self.label_frame, text=display_name)
         lbl.grid(row=row, column=0, sticky="e")
 
         value = self.metadata_model.get_value(name)
@@ -436,10 +606,10 @@ class MetadataFormFrame(ttk.Frame):
 
         # Add button to enable the removal of the field if it is optional.
         if name in self.metadata_model.optional_item_names:
-            btn = tk.Button(self.label_frame, text="Remove")
+            btn = ttk.Button(self.label_frame, text="Remove")
             btn._name_to_clear = name
             btn.bind("<Button-1>", self.master.deselect_optional_metadata)
-            btn.grid(row=row, column=2)
+            btn.grid(row=row, column=2, sticky="w")
 
         # Highlight input field if the value is invalid.
         background = "white"
@@ -448,7 +618,7 @@ class MetadataFormFrame(ttk.Frame):
             for issue in self.metadata_model.issues(name):
                 row = row + 1
                 issue_lbl = ttk.Label(self.label_frame, text=issue)
-                issue_lbl.grid(row=row, column=1, columnspan=2, sticky="w")
+                issue_lbl.grid(row=row, column=1, columnspan=2, sticky="nw")
         self.entries[name].config({"background": background})
 
         return row + 1
@@ -465,15 +635,20 @@ class MetadataFormFrame(ttk.Frame):
             row = self.setup_input_field(row, name)
 
 
-class NewDataSetWindow(tk.Toplevel):
-    """Preferences window."""
+class NewDataSetFrame(ttk.Frame):
+    """New dataset frame."""
 
-    def __init__(self, master):
+    def __init__(self, master, root):
         super().__init__(master)
         logger.info("Initialising {}".format(self))
-        self.title("New dataset")
         self.focus_set()
-        self.root = master
+        self.master = master
+        self.root = root
+
+        # Make sure that the GUI expands/shrinks when the window is resized.
+        self.columnconfigure(0, weight=1)
+        self.columnconfigure(1, weight=1)
+        self.rowconfigure(1, weight=1)
 
         self.metadata_schema_list_model = MetadataSchemaListModel()
         self.metadata_schema_list_model.put_metadata_schema_directory(METADATA_SCHEMAS_DIR)  # NOQA
@@ -481,21 +656,11 @@ class NewDataSetWindow(tk.Toplevel):
         default_metadata_model = self.metadata_schema_list_model.get_metadata_model("basic")  # NOQA
 
         self.proto_dataset_model = ProtoDataSetModel()
-        self.proto_dataset_model.set_base_uri_model(self.master.base_uri_model)
+        self.proto_dataset_model.set_base_uri_model(self.root.base_uri_model)
         self.proto_dataset_model.set_metadata_model(default_metadata_model)
 
-        # If we use the code below for styling the NewDataSetConfigFrame,
-        # OptionalMetadataFrame, MetadataFormFrame and ttk.Button need to have
-        # mainframe as their first argument. However, some of these currently
-        # call select_optional_metadata, deselect_optional_metadata, create
-        # and refresh. That stops working in that case as they are not on the
-        # mainframe object. The only reason to change to the below is to do
-        # with background styling of the tk.Toplevel window.
-#       mainframe = ttk.Frame(self)
-#       mainframe.grid(row=0, column=0, sticky="nwes")
-
         self.new_dataset_config_frame = NewDataSetConfigFrame(self, self.root)
-        self.new_dataset_config_frame.grid(row=0, column=0, columnspan=2)
+        self.new_dataset_config_frame.grid(row=0, column=0, columnspan=2, sticky="ew")  # NOQA
 
         self.optional_metadata_frame = OptionalMetadataFrame(self, self.root)
         self.optional_metadata_frame.grid(row=1, column=0, sticky="nsew")
@@ -531,7 +696,7 @@ class NewDataSetWindow(tk.Toplevel):
         else:
             self.create_btn.config(state=tk.NORMAL)
             self.progressbar.destroy()
-            self.master.refresh()
+            self.root.refresh()
 
     def _run_create(self):
         try:
@@ -539,7 +704,16 @@ class NewDataSetWindow(tk.Toplevel):
         except Exception as e:
             logger.warning("Dataset creation exception: {}".format(e))
             mb.showwarning("Failed to create dataset", e)
+            self.focus_set()
+            return
         logger.info("Finished dataset creation")
+        mb.showinfo(
+            "Dataset created",
+            message="{} dataset created at: {}".format(
+                self.proto_dataset_model._name,
+                self.proto_dataset_model._uri
+            )
+        )
 
     def create(self):
         # Need to check this as _yield_path_handle_tuples will fail if the
@@ -549,6 +723,7 @@ class NewDataSetWindow(tk.Toplevel):
                 "Failed to create dataset",
                 "Input directory has not been set"
             )
+            self.focus_set()
             return
 
         if self.root.base_uri_model.get_base_uri() is None:
@@ -556,6 +731,7 @@ class NewDataSetWindow(tk.Toplevel):
                 "Failed to create dataset",
                 "Local base URI has not been configured. Configure it in the preferences."  # NOQA
             )
+            self.focus_set()
             return
 
         # The number of items is needed for the progress bar.
@@ -579,6 +755,124 @@ class NewDataSetWindow(tk.Toplevel):
         self.metadata_form_frame.refresh()
 
 
+class NewDataSetWindow(tk.Toplevel):
+    """New dataset window."""
+    def __init__(self, master):
+        super().__init__(master)
+        self.title("Create dataset")
+        logger.info("Initialising {}".format(self))
+
+        # Make sure that the GUI expands/shrinks when the window is resized.
+        self.columnconfigure(0, weight=1)
+        self.rowconfigure(0, weight=1)
+
+        new_dataset_frame = NewDataSetFrame(self, master)
+        new_dataset_frame.grid(row=0, column=0, sticky="nwes")
+
+
+class EditMetadataFrame(ttk.Frame):
+    """Edit metadata frame."""
+
+    def __init__(self, master, root, dataset_uri):
+        super().__init__(master)
+        logger.info("Initialising {}".format(self))
+        self.focus_set()
+        self.master = master
+        self.root = root
+
+        self.dataset_model = DataSetModel()
+        self.dataset_model.load_dataset(dataset_uri)
+
+        # Mark any optional metadata items with data as selected.
+        for optional_item_name in self.metadata_model.optional_item_names:
+            value = self.metadata_model.get_value(optional_item_name)
+            if value is not None:
+                self.metadata_model.select_optional_item(optional_item_name)
+
+        self.optional_metadata_frame = OptionalMetadataFrame(self, self.root)
+        self.optional_metadata_frame.grid(row=0, column=0, sticky="nsew")
+
+        self.metadata_form_frame = MetadataFormFrame(self, self.root)
+        self.metadata_form_frame.grid(row=0, column=1, sticky="nsew")
+
+        self.cancel_btn = ttk.Button(self, text="Cancel", command=self.cancel)
+        self.cancel_btn.grid(row=1, column=0)
+
+        self.update_btn = ttk.Button(self, text="Update", command=self.update)
+        self.update_btn.grid(row=1, column=1)
+
+    @property
+    def proto_dataset_model(self):
+        # This is a hack to make it work with OptionalMetadataFrame.
+        return self.dataset_model
+
+    @property
+    def metadata_model(self):
+        return self.dataset_model.metadata_model
+
+    def update(self):
+        logger.info("Updating metadata for {}".format(self.dataset_model.name))
+        active_index = self.root.dataset_list_model.active_index
+        self.dataset_model.update_metadata()
+        self.root.refresh()
+        self.root.dataset_list_frame.update_selected_dataset(active_index)
+        self.root.edit_metadata_window = None
+        self.master.destroy()
+
+    def cancel(self):
+        logger.info("Not updating metadata {}".format(self.dataset_model.name))
+        self.root.edit_metadata_window = None
+        self.master.destroy()
+
+    def select_optional_metadata(self, event):
+        widget = event.widget
+        try:
+            index = int(widget.curselection()[0])
+        except IndexError:
+            return
+        name = widget.get(index)
+        logger.info(f"Selected optional metadata: {name}")
+        self.dataset_model.metadata_model.select_optional_item(name)
+        self.refresh()
+        self.metadata_form_frame.entries[name].focus_set()
+
+    def deselect_optional_metadata(self, event):
+        widget = event.widget
+        name = widget._name_to_clear
+        logger.info(f"Deselected optional metadata: {name}")
+        self.dataset_model.metadata_model.deselect_optional_item(name)
+        self.refresh()
+
+    def refresh(self):
+        self.optional_metadata_frame.refresh()
+        self.metadata_form_frame.refresh()
+
+
+class EditMetadataWindow(tk.Toplevel):
+    """Edit metadata window."""
+    def __init__(self, master, dataset_uri):
+        super().__init__(master)
+
+        self.root = master
+
+        # Stop user being able to interact with any other window.
+        self.grab_set()
+
+        # Implement custom behaviour when closing the window.
+        # Needed to set the App.edit_metadata_window to None.
+        self.protocol("WM_DELETE_WINDOW", self.dismiss)
+
+        ds_name = self.root.dataset_list_model.get_active_name()
+        self.title("Edit metadata: {}".format(ds_name))
+        logger.info("Initialising {}".format(self))
+        edit_metadata_frame = EditMetadataFrame(self, master, dataset_uri)
+        edit_metadata_frame.grid(row=0, column=0, sticky="nwes")
+
+    def dismiss(self):
+        self.root.edit_metadata_window = None
+        self.destroy()
+
+
 class PreferencesWindow(tk.Toplevel):
     """Preferences window."""
 
@@ -586,24 +880,46 @@ class PreferencesWindow(tk.Toplevel):
         super().__init__(master)
         self.title("Preferences")
 
+        # Implement custom behaviour when closing the window.
+        # Needed to set the App.preferences_window to None.
+        self.protocol("WM_DELETE_WINDOW", self.dismiss)
+
         logger.info("Initialising {}".format(self))
         self.root = master
         mainframe = ttk.Frame(self)
         mainframe.grid(row=0, column=0, sticky="nwes")
-        label_frame = ttk.LabelFrame(mainframe, text="Local base URI directory")  # NOQA
-        label_frame.grid(row=0, column=0,)
+        self.label_frame = ttk.LabelFrame(mainframe, text="Local base URI")  # NOQA
+        self.label_frame.grid(row=0, column=0,)
         self.local_base_uri_directory = tk.StringVar()
         self.local_base_uri_directory.set(
             self.root.base_uri_model.get_base_uri()
         )
-        label = ttk.Label(label_frame, textvar=self.local_base_uri_directory)
-        button = ttk.Button(
-            label_frame,
+        self.refresh()
+
+    def dismiss(self):
+        self.root.preferences_window = None
+        self.destroy()
+
+    def _setup_base_uri_directory_input_field(self):
+        self.help_lbl = ttk.Label(
+            self.label_frame,
+            text="Directory where datasets are stored."
+        )
+        self.lbl = ttk.Label(self.label_frame, text="Base URI")
+        self.entry = ttk.Entry(
+            self.label_frame,
+            width=len(self.root.base_uri_model.get_base_uri())
+        )
+        self.entry.insert(0, self.root.base_uri_model.get_base_uri())
+        self.entry.configure(state="readonly")
+        self.button = ttk.Button(
+            self.label_frame,
             text="Select local base URI directory",
             command=self.select_local_base_uri_directory
         )
-        label.grid(row=0, column=0)
-        button.grid(row=1, column=0)
+        self.help_lbl.grid(row=0, column=0, columnspan=2, sticky="ew")
+        self.entry.grid(row=1, column=0, sticky="ew")
+        self.button.grid(row=1, column=1)
 
     def select_local_base_uri_directory(self):
         base_uri_directory = fd.askdirectory(
@@ -617,8 +933,17 @@ class PreferencesWindow(tk.Toplevel):
                 base_uri_directory
             )
         )
+        self.refresh()
         self.root.refresh()
 
+    def refresh(self):
+        logger.info("Refreshing {}".format(self))
+        for widget in self.label_frame.winfo_children():
+            logger.info("Destroying widget: {}".format(widget))
+            widget.destroy()
+
+        self._setup_base_uri_directory_input_field()
+
 
 class NewDataSetProgressBar(ttk.Progressbar):
 
@@ -645,6 +970,10 @@ class App(tk.Tk):
         logger.info("Initialising dtool-gui")
         self.title("dtool")
 
+        self.preferences_window = None
+        self.edit_metadata_window = None
+        self.active_dataset_metadata_supported = False
+
         # Make sure that the GUI expands/shrinks when the window is resized.
         self.columnconfigure(0, weight=1)
         self.rowconfigure(0, weight=1)
@@ -656,9 +985,9 @@ class App(tk.Tk):
 
         # Configure the models.
         self.dataset_list_model.set_base_uri_model(self.base_uri_model)
-        if len(self.dataset_list_model.names) > 0:
-            first_uri = self.dataset_list_model.get_uri(0)
-            self.load_dataset(first_uri)
+        dataset_uri = self.dataset_list_model.get_active_uri()
+        if dataset_uri is not None:
+            self.load_dataset(dataset_uri)
 
         # Determine the platform.
         self.platform = self.tk.call("tk", "windowingsystem")
@@ -728,13 +1057,17 @@ class App(tk.Tk):
 
         # Make sure that the GUI expands/shrinks when the window is resized.
         self.mainframe.columnconfigure(0, weight=1)
+        self.mainframe.columnconfigure(2, weight=1)
         self.mainframe.rowconfigure(0, weight=1)
 
         self.dataset_list_frame = DataSetListFrame(self.mainframe, self)
         self.dataset_list_frame.grid(row=0, column=0, sticky="nsew")
 
+        # Add vertical separator between dataset list and dataset views.
+        ttk.Separator(self.mainframe, orient=tk.VERTICAL).grid(row=0, column=1, sticky="ns")  # NOQA
+
         self.dataset_frame = DataSetFrame(self.mainframe, self)
-        self.dataset_frame.grid(row=0, column=1, sticky="new")
+        self.dataset_frame.grid(row=0, column=2, sticky="nsew")
 
         # Ask the user to configure the local base URI if it has not been configured.  # NOQA
         if self.base_uri_model.get_base_uri() is None:
@@ -781,10 +1114,20 @@ class App(tk.Tk):
     def edit_metadata(self):
         """Open window with form to edit a dataset's metadata."""
         logger.info(self.edit_metadata.__doc__)
-        mb.showinfo(
-            "Work in progress...",
-            "It is not possible to edit an existing dataset's metadata yet."
-        )
+        if self.edit_metadata_window is None:
+            if self.active_dataset_metadata_supported:
+                self.edit_metadata_window = EditMetadataWindow(
+                    self,
+                    self.dataset_list_model.get_active_uri()
+                )
+            else:
+                logger.info("Can't edit metadata (contains unsupported metadata types) {}.".format(self.dataset_list_model.get_active_uri()))  # NOQA
+                mb.showinfo(
+                    "Can't edit dataset",
+                    message="Dataset contains unsupported metadata types. Try using the dtool CLI to edit its metadata."  # NOQA
+                )
+        else:
+            self.edit_metadata_window.focus_set()
 
     def _quit_event(self, event):
         self.quit()
@@ -799,14 +1142,19 @@ class App(tk.Tk):
     def show_perferences_window(self):
         """Show the preferences window."""
         logger.info(self.show_perferences_window.__doc__)
-        PreferencesWindow(self)
+        if self.preferences_window is None:
+            self.preferences_window = PreferencesWindow(self)
+        else:
+            self.preferences_window.focus_set()
 
     def load_dataset(self, dataset_uri):
         """Load dataset and deal with UnsupportedTypeError exceptions."""
         try:
             self.dataset_model.load_dataset(dataset_uri)
+            self.active_dataset_metadata_supported = True
         except UnsupportedTypeError:
             logging.warning("Dataset contains unsupported metadata type")
+            self.active_dataset_metadata_supported = False
 
     def refresh(self):
         """Refreshing all frames."""
